@@ -1,6 +1,8 @@
+#include <stdlib.h>
 #include <stdio.h>
 #include "t_pwd.h"
 
+int
 main(argc, argv)
 int argc;
 char ** argv;
