@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997-2001  The Stanford SRP Authentication Project
+ * Copyright (c) 1997-2004  The Stanford SRP Authentication Project
  * All Rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining
@@ -39,8 +39,8 @@
  *    of this copyright notice and list of conditions.
  */
 
+#include <stdlib.h>
 #include <stdio.h>
-#include "t_defines.h"
 #include "t_pwd.h"
 #include "t_client.h"
 #include "t_server.h"
@@ -48,24 +48,23 @@
 
 /* Test SRP authentication by simulating both sides of a session */
 
+int vflag = 0;
+
 int
-main(argc, argv)
-     int argc;
-     char ** argv;
+do_test_pindex(int pindex)
 {
   /* classic API */
   struct t_preconf * pc;
   struct t_confent ce;
   struct t_pwent * ppwe;
-  struct t_pw tpw;
+  struct t_pw * tpw;
   struct t_server * ts;
   struct t_client * tc;
   struct t_num * A;
   struct t_num * B;
   int i;
-  int pindex;
-  int vflag = 0;
-  char hexbuf[MAXHEXPARAMLEN];
+  char * hexbuf;
+  int hblen;
   unsigned char * ckey;
   unsigned char * skey;
   unsigned char * cresp;
@@ -84,26 +83,12 @@ main(argc, argv)
   /* cross test */
   struct t_num tmpB;
 
-  pindex = t_getprecount() - 1;
-
-  while(--argc > 0 && **++argv == '-') {
-    if(strcmp(*argv, "-v") == 0)
-      ++vflag;
-    else if((*argv)[1] >= '0' && (*argv)[1] <= '9')
-      pindex = atoi(*argv + 1);
-    else {
-      fprintf(stderr, "Usage: srptest [-v] [-index]\n");
-      fprintf(stderr, "       where 'index' is a precompiled parameter index\n");
-      exit(1);
-    }
-  }
-
   printf("[server] initializing password: '%s'\n", pass);
 
   pc = t_getpreparam(pindex);	/* Use largest parameter set */
   if(pc == NULL) {
     fprintf(stderr, "Unable to load parameter set %d\n", pindex);
-    exit(2);
+    return -1;
   }
 
   ce.modulus.len = pc->modulus.len;
@@ -113,7 +98,11 @@ main(argc, argv)
   ce.index = i - 1;
 
   /* TODO: Add an official "empty" constructor for struct t_pw */
-  ppwe = t_makepwent(&tpw, user, pass, NULL, &ce);
+  tpw = t_newpw();
+  ppwe = t_makepwent(tpw, user, pass, NULL, &ce);
+
+  hblen = 2 * (ce.modulus.len < 32 ? 32 : ce.modulus.len) + 1;
+  hexbuf = (char *)malloc(hblen);
 
   printf("[server] modulus = %s\n",
 	 t_tohex(hexbuf, ce.modulus.data, ce.modulus.len));
@@ -124,6 +113,10 @@ main(argc, argv)
   printf("[server] salt = %s\n",
 	 t_tohex(hexbuf, ppwe->salt.data, ppwe->salt.len));
 
+  /* Skip classic API if modulus exceeds old limit */
+  if(ce.modulus.len > 256)
+    goto newtest;
+
   /* Begin classic API test */
   printf("\n*** Testing classic API ***\n\n");
 
@@ -176,6 +169,7 @@ main(argc, argv)
   if(i != 0)
     return i;
 
+newtest:
   /* Begin new API test */
   printf("\n*** Testing new API ***\n\n");
 
@@ -216,7 +210,7 @@ main(argc, argv)
     return 1;
   }
   if(vflag) {
-    BigIntegerToString(srpc->secret, hexbuf, 16);
+    BigIntegerToString(srpc->secret, hexbuf, hblen, 16);
     printf("[client private] a = %s\n", hexbuf);
   }
   printf("[client] sending A = %s\n\n", t_tohex(hexbuf, t1->data, t1->length));
@@ -227,19 +221,19 @@ main(argc, argv)
     return 1;
   }
   if(vflag) {
-    BigIntegerToString(srps->secret, hexbuf, 16);
+    BigIntegerToString(srps->secret, hexbuf, hblen, 16);
     printf("[server private] b = %s\n", hexbuf);
   }
   printf("[server] sending B = %s\n", t_tohex(hexbuf, t2->data, t2->length));
   ks = NULL;
   if(SRP_compute_key(srps, &ks, t1->data, t1->length) != SRP_SUCCESS) {
-    printf("SRP_compute_key failed\n");
+    printf("SRP_compute_key (server) failed\n");
     return 1;
   }
   if(vflag) {
-    BigIntegerToString(srps->u, hexbuf, 16);
+    BigIntegerToString(srps->u, hexbuf, hblen, 16);
     printf("[server] u = %s\n", hexbuf);
-    BigIntegerToString(srps->key, hexbuf, 16);
+    BigIntegerToString(srps->key, hexbuf, hblen, 16);
     printf("[server] raw key = %s\n", hexbuf);
   }
   printf("[server] session key = %s\n\n",
@@ -251,13 +245,13 @@ main(argc, argv)
   }
   kc = NULL;
   if(SRP_compute_key(srpc, &kc, t2->data, t2->length) != SRP_SUCCESS) {
-    printf("SRP_compute_key failed\n");
+    printf("SRP_compute_key (client) failed\n");
     return 1;
   }
   if(vflag) {
-    BigIntegerToString(srpc->u, hexbuf, 16);
+    BigIntegerToString(srpc->u, hexbuf, hblen, 16);
     printf("[client] u = %s\n", hexbuf);
-    BigIntegerToString(srpc->key, hexbuf, 16);
+    BigIntegerToString(srpc->key, hexbuf, hblen, 16);
     printf("[client] raw key = %s\n", hexbuf);
   }
   printf("[client] session key = %s\n",
@@ -294,6 +288,10 @@ main(argc, argv)
   if(i != 0)
     return i;
 
+  /* Skip cross-API test if modulus exceeds old limit */
+  if(ce.modulus.len > 256)
+    goto srp6test;
+
   /* Begin cross-API test */
   printf("\n*** Cross-testing APIs ***\n\n");
 
@@ -330,19 +328,19 @@ main(argc, argv)
     return 1;
   }
   if(vflag) {
-    BigIntegerToString(srps->secret, hexbuf, 16);
+    BigIntegerToString(srps->secret, hexbuf, hblen, 16);
     printf("[server private] b = %s\n", hexbuf);
   }
   printf("[server] sending B = %s\n", t_tohex(hexbuf, t2->data, t2->length));
   ks = cstr_new();
   if(SRP_compute_key(srps, &ks, A->data, A->len) != SRP_SUCCESS) {
-    printf("SRP_compute_key failed\n");
+    printf("SRP_compute_key (server) failed\n");
     return 1;
   }
   if(vflag) {
-    BigIntegerToString(srps->u, hexbuf, 16);
+    BigIntegerToString(srps->u, hexbuf, hblen, 16);
     printf("[server] u = %s\n", hexbuf);
-    BigIntegerToString(srps->key, hexbuf, 16);
+    BigIntegerToString(srps->key, hexbuf, hblen, 16);
     printf("[server] raw key = %s\n", hexbuf);
   }
   printf("[server] session key = %s\n\n",
@@ -378,5 +376,178 @@ main(argc, argv)
   cstr_free(ks);
   cstr_free(t2);
 
+  if(i != 0)
+    return i;
+
+srp6test:
+  /* Begin SRP-6 test (new API) */
+  printf("\n*** Testing SRP-6a ***\n\n");
+
+  t1 = NULL;
+  printf("[client] sending username '%s'\n\n", user);
+
+  printf("[server] initializing session\n");
+  srps = SRP_new(SRP6a_server_method());
+  if(SRP_set_username(srps, user) < 0) {
+    printf("SRP_set_username failed\n");
+    return 1;
+  }
+  if(SRP_set_params(srps, ce.modulus.data, ce.modulus.len,
+		    ce.generator.data, ce.generator.len,
+		    ppwe->salt.data, ppwe->salt.len) < 0) {
+    printf("SRP_set_params failed\n");
+    return 1;
+  }
+  if(SRP_set_auth_password(srps, pass) < 0) {
+    printf("SRP_set_authenticator failed\n");
+    return 1;
+  }
+
+  printf("[client] initializing session\n");
+  srpc = SRP_new(SRP6a_client_method());
+  if(SRP_set_username(srpc, user) != SRP_SUCCESS) {
+    printf("SRP_set_username failed\n");
+    return 1;
+  }
+  if(SRP_set_params(srpc, ce.modulus.data, ce.modulus.len,
+		    ce.generator.data, ce.generator.len,
+		    ppwe->salt.data, ppwe->salt.len) != SRP_SUCCESS) {
+    printf("SRP_set_params failed\n");
+    return 1;
+  }
+  if(SRP_gen_pub(srpc, &t1) != SRP_SUCCESS) {
+    printf("SRP_gen_pub failed\n");
+    return 1;
+  }
+  if(vflag) {
+    BigIntegerToString(srpc->secret, hexbuf, hblen, 16);
+    printf("[client private] a = %s\n", hexbuf);
+  }
+  printf("[client] sending A = %s\n\n", t_tohex(hexbuf, t1->data, t1->length));
+
+  t2 = NULL;
+  if(SRP_gen_pub(srps, &t2) != SRP_SUCCESS) {
+    printf("SRP_gen_pub failed\n");
+    return 1;
+  }
+  if(vflag) {
+    BigIntegerToString(srps->secret, hexbuf, hblen, 16);
+    printf("[server private] b = %s\n", hexbuf);
+  }
+  printf("[server] sending B = %s\n", t_tohex(hexbuf, t2->data, t2->length));
+  ks = NULL;
+  if(SRP_compute_key(srps, &ks, t1->data, t1->length) != SRP_SUCCESS) {
+    printf("SRP_compute_key (server) failed\n");
+    return 1;
+  }
+  if(vflag) {
+    BigIntegerToString(srps->u, hexbuf, hblen, 16);
+    printf("[server] u = %s\n", hexbuf);
+    BigIntegerToString(srps->key, hexbuf, hblen, 16);
+    printf("[server] raw key = %s\n", hexbuf);
+  }
+  printf("[server] session key = %s\n\n",
+	 t_tohex(hexbuf, ks->data, ks->length));
+
+  if(SRP_set_auth_password(srpc, pass) != SRP_SUCCESS) {
+    printf("SRP_set_authenticator failed\n");
+    return 1;
+  }
+  kc = NULL;
+  if(SRP_compute_key(srpc, &kc, t2->data, t2->length) != SRP_SUCCESS) {
+    printf("SRP_compute_key (client) failed\n");
+    return 1;
+  }
+  if(vflag) {
+    BigIntegerToString(srpc->u, hexbuf, hblen, 16);
+    printf("[client] u = %s\n", hexbuf);
+    BigIntegerToString(srpc->key, hexbuf, hblen, 16);
+    printf("[client] raw key = %s\n", hexbuf);
+  }
+  printf("[client] session key = %s\n",
+	 t_tohex(hexbuf, kc->data, kc->length));
+  if(SRP_respond(srpc, &kc) != SRP_SUCCESS) {
+    printf("SRP_respond failed\n");
+    return 1;
+  }
+  printf("[client] sending client proof = %s\n\n",
+	 t_tohex(hexbuf, kc->data, kc->length));
+
+  i = SRP_verify(srps, kc->data, kc->length);
+  printf("[server] verify status = %d (%s)\n", i,
+	 (i == SRP_SUCCESS) ? "success" : "failure");
+  if(i == SRP_SUCCESS) {
+    if(SRP_respond(srps, &ks) != SRP_SUCCESS) {
+      printf("SRP_respond failed\n");
+      return 1;
+    }
+    printf("[server] sending server proof = %s\n\n",
+	   t_tohex(hexbuf, ks->data, ks->length));
+
+    i = SRP_verify(srpc, ks->data, ks->length);
+    printf("[client] verify status = %d (%s)\n", i,
+	   (i == SRP_SUCCESS) ? "success" : "failure");
+  }
+  SRP_free(srpc);
+  SRP_free(srps);
+  cstr_free(t1);
+  cstr_free(t2);
+  cstr_free(kc);
+  cstr_free(ks);
+
+  free(hexbuf);
+
+  t_closepw(tpw);
+
+  return i;
+}
+
+int
+main(argc, argv)
+     int argc;
+     char ** argv;
+{
+  int i;
+  int pindex = -1;
+
+  SRP_initialize_library();
+
+  while(--argc > 0 && **++argv == '-') {
+    if(strcmp(*argv, "-v") == 0)
+      ++vflag;
+    else if((*argv)[1] >= '0' && (*argv)[1] <= '9')
+      pindex = atoi(*argv + 1);
+    else if(strcmp(*argv, "-engine") == 0) {
+      if(--argc > 0 && *++argv != NULL) {
+	if(!SRP_OK(SRP_use_engine(*argv))) {
+	  fprintf(stderr, "Unable to use engine '%s'\n", *argv);
+	  exit(2);
+	}
+      }
+      else {
+	fprintf(stderr, "No engine name supplied - exiting\n");
+	exit(1);
+      }
+    }
+    else {
+      fprintf(stderr, "Usage: srptest [-v] [-index] [-engine e]\n");
+      fprintf(stderr, "       where 'index' is a precompiled parameter index\n");
+      exit(1);
+    }
+  }
+
+  if(pindex < 0) {
+    for(pindex = 0; pindex < t_getprecount(); ++pindex) {
+      i = do_test_pindex(pindex);
+      if(i != 0)
+	break;
+    }
+  }
+  else {
+    i = do_test_pindex(pindex);
+  }
+
+  SRP_finalize_library();
+
   return i;
 }
