@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997-2003  The Stanford SRP Authentication Project
+ * Copyright (c) 1997-2004  The Stanford SRP Authentication Project
  * All Rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining
@@ -72,7 +72,12 @@ typedef void * BigIntegerModAccel;
 #  define GMP_IMPEXP 1
 # endif
 #elif defined(TOMMATH)
-# include "tommath.h"
+# ifdef TOMCRYPT
+   /* as of v0.96 */
+#  include "ltc_tommath.h"
+# else
+#  include "tommath.h"
+# endif
 typedef mp_int * BigInteger;
 typedef void * BigIntegerCtx;
 typedef void * BigIntegerModAccel;
@@ -234,6 +239,20 @@ BigIntegerToCstr(BigInteger x, cstr * out)
   return BIG_INTEGER_SUCCESS;
 }
 
+BigIntegerResult
+BigIntegerToCstrEx(BigInteger x, cstr * out, int len)
+{
+  int n;
+  if(cstr_set_length(out, len) < 0)
+    return BIG_INTEGER_ERROR;
+  n = BigIntegerToBytes(x, out->data, len);
+  if(n < len) {
+    memmove(out->data + (len - n), out->data, n);
+    memset(out->data, 0, len - n);
+  }
+  return BIG_INTEGER_SUCCESS;
+}
+
 BigIntegerResult
 BigIntegerToHex(src, dest, destlen)
      BigInteger src;
