@@ -104,13 +104,16 @@ usage()
 #else
 	    "[-r] ",
 #endif
+#ifdef KRB5
+	    "[-A] ",
+#endif /* KRB5 */
 #ifdef FWD_X
 	    "[-D]\n\t",
 #else /* FWD_X */
 	    "\n\t",
 #endif /* FWD_X */
 #ifdef TLS
-	    "[-T cert=FILE] [-T key=FILE] [-T crlfile=FILE] [-T crldir=DIR]\n\t[-T cipher=STRING] ",
+	    "[-T cert=FILE] [-T key=FILE] [-T crlfile=FILE] [-T crldir=DIR]\n\t[-T cipher=STRING] [-T CApath=DIR] [-T CAfile=FILE]\n\t",
 #else /* TLS */
 	    "",
 #endif /* TLS */
@@ -149,6 +152,9 @@ main(argc, argv)
 #ifdef ENCRYPTION
         extern int auth_enable_encrypt;
 #endif /* ENCRYPTION */
+#ifdef KRB5
+	extern int krb5_use_addresses;
+#endif /* KRB5 */
 
 	tninit();		/* Clear out things */
 #if	defined(CRAY) && !defined(__STDC__)
@@ -167,7 +173,7 @@ main(argc, argv)
 	rlogin = (strncmp(prompt, "rlog", 4) == 0) ? '~' : _POSIX_VDISABLE;
 	autologin = -1;
 
-	while ((ch = getopt(argc, argv, "8EKLS:X:acde:fFk:l:n:rt:xT:D")) != EOF) {
+	while ((ch = getopt(argc, argv, "8EKLS:X:acde:fFk:l:n:rt:xT:DA")) != EOF) {
 		switch(ch) {
 		case '8':
 			eight = 3;	/* binary output and input */
@@ -332,6 +338,11 @@ main(argc, argv)
 			fwdx_enable_flag = 0;
 			break;
 #endif  /* FWD_X */
+#ifdef KRB5
+	        case 'A':
+			krb5_use_addresses = 0;
+			break;
+#endif	/* KRB5 */
 
 		case '?':
 		default:
